#include "reflect.h"

namespace Msp {
namespace GL {
namespace SL {

LocationCounter::LocationCounter():
	r_count(0)
{ }

void LocationCounter::visit(BasicTypeDeclaration &basic)
{
	r_count = basic.kind==BasicTypeDeclaration::MATRIX ? basic.size>>16 : 1;
}

void LocationCounter::visit(ImageTypeDeclaration &)
{
	r_count = 1;
}

void LocationCounter::visit(StructDeclaration &strct)
{
	unsigned total = 0;
	for(NodeList<Statement>::const_iterator i=strct.members.body.begin(); i!=strct.members.body.end(); ++i)
	{
		r_count = 1;
		(*i)->visit(*this);
		total += r_count;
	}
	r_count = total;
}

void LocationCounter::visit(VariableDeclaration &var)
{
	r_count = 1;
	if(var.type_declaration)
		var.type_declaration->visit(*this);
	if(var.array)
		if(const Literal *literal = dynamic_cast<const Literal *>(var.array_size.get()))
			if(literal->value.check_type<int>())
				r_count *= literal->value.value<int>();
}

} // namespace SL
} // namespace GL
} // namespace Msp
